import config from "../../config";
import {
	toastHide,
	toastMessage
} from "./toastUtil";
import {
	printInfo,
	printDebug,
	printError,
} from "./printUtil.js"
import {
	appendParam,
} from "./util";
import {
	PAKEAGE_NAME
} from "../../utils/utils";
import {
	addNormalNotificationObserver,
	removeNotificationObserver
} from "./notificationCenter.js"
import {
	KEY_NOTIFICATION_LOGIN_SUCCESS
} from "../../static/keys/notification-keys.js"
import {
	showLoginView,
	logout
} from "../services/userServices.js"
import {
	readStorage
} from "./storageUtil";
import {
	readToken
} from "../services/userServices.js"

const WHITE_LIST = [] // 免Token白名单

/**
 * token白名单
 * @param {Object} token token
 * @param {Object} url 请求地址
 */
function checkTokenWhiteList(token, url, urlModule) {
	return true;
	// if (token) return true;
	// for (let index = 0; index < WHITE_LIST.length; index++) {
	// 	if (url.startsWith(WHITE_LIST[index])) {
	// 		return true;
	// 	}
	// }
	// return false;
}

/**
 * apiRequest
 */
let apiRequest = function({
	url,
	urlModule = config[`BASE_URL_MODULE`], // 路径识别模块
	data,
	method = 'GET',
	header,
	callback,
	complete
}) {
	let originQuery = {
		...arguments[0]
	};
	let option = {}; // 请求参数
	// url 处理
	let handlerUrl = `${config["BASE_URL"]}${urlModule?urlModule:""}${url}`;
	// if (handlerUrl.indexOf("?") == -1) {
	// 	handlerUrl = `${handlerUrl}?t=${new Date().getTime()}`;
	// } else {
	// 	handlerUrl = `${handlerUrl}&t=${new Date().getTime()}`;
	// }
	option = {
		url: handlerUrl,
		method
	}
	// data处理
	if (data) {
		if (typeof data == 'object' && !(data instanceof Array)) { // 对于非 array 的 object 对象， 判断并移除为空的参数
			let dataResult = {};
			Object.keys(data).forEach(key => {
				if (data[key] != null && data[key] != undefined) {
					dataResult[key] = data[key]
				}
			})
			option.data = dataResult; // 请求数据
		} else { // 其余对象照常插入
			option.data = data;
		}
	}
	console.log('123')
	// token处理
	let token = readToken();
	if (!checkTokenWhiteList(token, url, urlModule)) {
		if (typeof originQuery.callback == 'function') originQuery.callback(false, null);
		if (typeof originQuery.complete == 'function') originQuery.complete();
		return;
	}

	let uniChannel = 'wechat';
	// #ifdef MP-KUAISHOU
	uniChannel = 'kuaishou';
	// #endif

	// #ifdef MP-TOUTIAO
	uniChannel = 'douyin';
	// #endif

	option.header = {
		pkgName: PAKEAGE_NAME,
		token: `${token}`,
		proChannel: uniChannel
	}
	// header处理
	if (header) {
		option.header = {
			...option.header,
			...header
		}
	}
	console.log('123')
	printInfo("start Req", option);
	let requestTask = uni.request({
		...option,
		success: (res) => {
			successHandler(originQuery, requestTask, res);
		},
		fail: (error) => {
			failHandler(originQuery, requestTask, error);
		},
		complete: (res) => {
			completeHandler(originQuery, requestTask)
		}
	})
	return requestTask;
}

function apiGET({
	url,
	urlModule = config[`BASE_URL_MODULE`],
	data,
	header,
	callback,
	complete
}) {
	return apiRequest({
		url,
		urlModule,
		data,
		header,
		method: 'GET',
		callback,
		complete
	});
}

function apiPOST({
	url,
	urlModule = config[`BASE_URL_MODULE`],
	data,
	header,
	callback,
	complete
}) {
	return apiRequest({
		url,
		urlModule,
		data,
		header,
		method: 'POST',
		callback,
		complete
	});
}

function apiPUT({
	url,
	urlModule = config[`BASE_URL_MODULE`],
	data,
	header,
	callback,
	complete
}) {
	return apiRequest({
		url,
		urlModule,
		data,
		header,
		method: 'PUT',
		callback,
		complete
	});
}

function apiDELETE({
	url,
	urlModule = config[`BASE_URL_MODULE`],
	data,
	header,
	callback,
	complete
}) {
	return apiRequest({
		url,
		urlModule,
		data,
		header,
		method: 'DELETE',
		callback,
		complete
	});
}

function apiUPLOAD({
	url,
	urlModule = config[`BASE_URL_MODULE`],
	method = "UPLOAD",
	filePath,
	name = "file",
	urlData = {},
	formData = {},
	header,
	callback,
	complete
}) {
	let originQuery = {
		...arguments[0]
	};
	let option = {}; // 请求参数
	if (filePath) {
		option.filePath = filePath
	}
	if (name) {
		option.name = name
	}
	if (formData) {
		option.formData = formData
	}
	let token = readToken()
	if (!checkTokenWhiteList(token, url, urlModule)) {
		if (typeof originQuery.callback == 'function') originQuery.callback(false, null);
		if (typeof originQuery.complete == 'function') originQuery.complete();
		return;
	}
	option.url = `${config['BASE_URL']}${urlModule?urlModule:""}${url}`;
	if (urlData) {
		option.url = `${option.url}${appendParam(urlData)}`;
	}
	option.url = `${option.url}?t=${new Date().getTime()}`; // 路径

	option.header = {
		pkgName: PAKEAGE_NAME,
		token: `${token}`
	}
	if (header) {
		option.header = {
			chartset: "utf-8",
			"content-type": "multipart/form-data",
			...option.header,
			...header
		}
	} // header
	console.log(JSON.stringify(option))
	let requestTask = uni.uploadFile({
		...option,
		success: (res) => {
			successHandler(originQuery, requestTask, res);
		},
		fail: (error) => {
			failHandler(originQuery, requestTask, error)
		},
		complete: () => {
			completeHandler(originQuery, requestTask);
		}
	})
	return requestTask;
}

/**
 * 请求成功处理
 * @param {Object} originQuery 原始请求
 * @param {Object} requestTask 请求任务
 * @param {Object} res 返回数据
 */
function successHandler(originQuery, requestTask, res) {
	let tokenExpireFn = () => {
		toastHide();
		printDebug("请求结果", originQuery.url, res.data.msg);

		addNormalNotificationObserver(KEY_NOTIFICATION_LOGIN_SUCCESS, (info) => {
			if (originQuery.method == 'UPLOAD') {
				apiUPLOAD(originQuery);
			} else {
				apiRequest(originQuery);
			}
			removeNotificationObserver(KEY_NOTIFICATION_LOGIN_SUCCESS, requestTask);
		}, requestTask);

		if (uni.getStorageSync('showLoginModal')) {
			return
		}
		uni.setStorageSync('showLoginModal', true)
		uni.showModal({
			title: "登录状态已过期",
			content: "当前登录状态已过期，请重新登录！",
			cancelText: "暂不登录",
			confirmText: "前往登录",
			success: (res) => {
				if (res.confirm) {
					showLoginView();
				} else {
					logout(() => {}, true);
				}
				uni.setStorageSync('showLoginModal', false)
			}
		})
	}
	if (res.statusCode != 200) {
		toastMessage(`服务器异常,${res.statusCode}`)
		printError("请求失败", originQuery.url, res.errMsg);
		if (typeof originQuery.callback == 'function') originQuery.callback(false, res.statusCode);
	} else if (res.code == 401) {
		tokenExpireFn();
	} else {
		if (typeof res.data == 'string') res.data = JSON.parse(res.data);
		if (res.data.status == 200) {
			printDebug("请求结果", originQuery.url, res.data.data);
			if (typeof originQuery.callback == 'function') originQuery.callback(true, res.data.data);
		} else if (res.data.status == 401) {
			tokenExpireFn()
			if (typeof originQuery.callback == 'function') originQuery.callback(false, res.data.msg);
		} else {
			toastMessage(res.data.message ? res.data.message : res.data.status ? res.data.status : "未知错误");
			printError("请求失败", originQuery.url, res.data.message);
			if (typeof originQuery.callback == 'function') originQuery.callback(false, res.data.message);
		}
	}
}

/**
 * 请求失败处理
 * @param {Object} originQuery 原始请求
 * @param {Object} requestTask 请求任务
 * @param {Object} error 错误
 */
function failHandler(originQuery, requestTask, error) {
	toastMessage("request fail!");
	printError("请求--失败", originQuery.url, error.errMsg);
	if (typeof originQuery.callback == 'function') originQuery.callback(false, error.errMsg);
}

/**
 * 请求完成处理
 * @param {Object} originQuery 原始请求
 * @param {Object} requestTask 请求任务
 */
function completeHandler(originQuery, requestTask) {
	if (typeof originQuery.complete == 'function') originQuery.complete();
}
/**
 *  * 
 * // ============= example ============ //
 *      request2Promise(apiGET, {
 *          url: `/order/detail/1570590978977517570`,
 *          data: {
 *              orderRunningId: "1570590978977517570"
 *          }
 *      }).then(res=>{
 *           console.log("promiseTest", res);
 *      }).catch(err=>{
 *          console.log("promiseTest error", err);
 *      });
 * 
 */
/**
 * request 转 promise
 * @param {Function|String} requestFn apiGET/apiPUT/apiPOST/apiDELETE/apiUPLOAD/apiRequest 函数以及支持此类方法相似参数的请求函数，或者 get,post,put,delete,request 字符串（不区分大小写），必须提供一个callback 参数， callback 接受 success 和 data 参数
 * @param {Object} params 参数，参考 apiGET/apiPUT/apiPOST/apiDELETE/apiUPLOAD/apiRequest 参数
 * @returns promise
 */
function request2Promise(requestFn, params) {
	return new Promise((resolve, reject) => {
		requestFn = get2PromiseRequestFn(requestFn);
		if (!requestFn) {
			toastMessage("请求方法错误");
			reject("请求方法错误");
		} else {
			requestFn({
				...params,
				callback: (success, data) => {
					if (success) {
						resolve(data);
					} else {
						reject(data);
					}
				}
			})
		}
	})
}

/**
 * 2Promise方法处理
 * @param {Function | String} requestFn function 或者 get|post|put|delete|upload|request
 */
function get2PromiseRequestFn(requestFn) {
	if (typeof requestFn == 'function') {
		return requestFn;
	}
	if (typeof requestFn == 'string') {
		requestFn = requestFn.toLowerCase();
		switch (requestFn) {
			case 'get':
				return apiGET;
			case 'post':
				return apiPOST;
			case 'put':
				return apiPUT;
			case "delete":
				return apiDELETE;
			case 'upload':
				return apiUPLOAD;
			case 'request':
				return apiRequest;
			default:
				return null;
		}
	}
	return null;
}

module.exports = {
	apiRequest,
	apiGET,
	apiPUT,
	apiPOST,
	apiDELETE,
	apiUPLOAD,
	request2Promise,
}