import Vue from 'vue'

/**
 * 姓名脱敏
 */
Vue.filter('filterName', value => {
	if (value.length == 2) {
		return value.substring(0, 1) + '*' //截取name 字符串截取第一个字符，
	} else if (value.length == 3) {
		return value.substring(0, 1) + "*" + value.substring(2, 3) //截取第一个和第三个字符
	} else if (value.length > 3) {
		return value.substring(0, 1) + "*" + '*' + value.substring(3, value.length) //截取第一个和大于第4个字符
	}
})
/**
 * Hash 截断
 */
Vue.filter('sliceHash', value => {
	if (!value) return '';
	return value.replace(/(\w{9})\w+(\w{3})/, '$1...$2');
})

/**
 * 元->元 金额转换 保留2位小数 并每隔3位用逗号分开
 * value: 20000.20
 * return: 20,000.20
 */
Vue.filter('formatMoney', value => {
	if (!value) return '0.00';
	let str = parseFloat(value).toFixed(4) + '';
	let intSum = str.substring(0, str.indexOf(".")).replace(/\B(?=(?:\d{3})+$)/g, ','); //取到整数部分
	let dot = str.substring(str.length, str.indexOf(".")) //取到小数部分搜索
	dot = (dot + '00').slice(0, 3)
	return intSum + dot;
})

/**
 * 分->元 金额转换 保留2位小数 并每隔3位用逗号分开
 * value: 2000000.20
 * return: 20,000.20
 */
Vue.filter('formatFractionMoney', value => {
	if (!value) return '0.00';
	let str = (parseFloat(value) / 100).toFixed(4) + '';
	let intSum = str.substring(0, str.indexOf(".")).replace(/\B(?=(?:\d{3})+$)/g, ','); //取到整数部分
	let dot = str.substring(str.length, str.indexOf(".")) //取到小数部分搜索
	dot = (dot + '00').slice(0, 3)
	return intSum + dot;
})

/**
 * 格式化显示身份证
 * value: xxxxxxxxxxxxxxxxxx
 * return: xxx xxx xxxx xxxx
 */
Vue.filter('formatIdCard', value => {
	value = value.replace(/\s/g, '');
	if (!value) return '';
	return value.replace(/(\d{3})(\d{3})(\d{4})(\d{4})(\d*)/g, "$1 $2 $3 $4 $5");
})

/**
 * 身份证脱敏
 * value: xxxxxxxxxxxxxxxxxx
 * return: xxx xxx xxxx xxxx
 */
Vue.filter('filterIdCard', value => {
	value = value.replace(/\s/g, '');
	if (!value) return '';
	return value.replace(/(\d{3})(\d{3})(\d{4})(\d{4})(\d*)/g, "$1*********$5");
})

/**
 * 格式化显示银行卡
 * value: xxxxxxxxxxxxxxxxxxxx
 * return: xxxx xxxx xxxx xxxx
 */
Vue.filter('formatBankCard', value => {
	value = value.replace(/\s/g, '');
	if (!value) return '';
	return value.replace(/(\d{4})(?=\d)/g, "$1 ");
})

/**
 * 脱敏银行卡
 * value: xxxxxxxxxxxxxxxxxxxx
 * return: xxxx xxxx xxxx xxxx
 */
Vue.filter('filterBankCard', value => {
	return value.substr(0, 3) + '******' + value.substr(value.length - 3)
})

/**
 * 格式化显示手机号
 * value: 12345678901
 * return: 123****8901
 */
Vue.filter('formatPhone2', value => {
	return value.substr(0, 3) + '****' + value.substr(value.length - 4)
})

/**
 * 格式化显示手机号
 * value: xxxxxxxxxxx
 * return: xxx xxxx xxxx
 */
Vue.filter('formatPhone', value => {
	value = value.replace(/\s/g, '');
	if (!value) return '';
	return value.replace(/(\d{3})(\d{4})(\d*)/g, "$1 $2 $3");
})

/**
 * 日期时间格式化
 * @param {Object} date
 * @param {Object} fmt
 */
Vue.filter('formatDate', function(dateTime, fmt) {
	let date = new Date(dateTime)
	if (/(y+)/.test(fmt)) {
		fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
	}
	let o = {
		'M+': (date.getMonth() + 1).toString().padStart(2, '0'),
		'd+': date.getDate().toString().padStart(2, '0'),
		'h+': date.getHours(),
		'm+': date.getMinutes(),
		's+': date.getSeconds()
	};
	for (let k in o) {
		if (new RegExp(`(${k})`).test(fmt)) {
			let str = o[k] + '';
			fmt = fmt.replace(RegExp.$1, (RegExp.$1.length === 1) ? str : str);
		}
	}
	return fmt;
})

/**
 * 格式化订单状态
 */
Vue.filter('formatOrderStatus', value => {
	if (value == 0) {
		return '待支付'
	} else if (value == 1) {
		return '待出库'
	} else if (value == 2) {
		return '配送中'
	} else if (value == 3) {
		return '已签收'
	} else if (value == 4) {
		return '退款'
	} else if (value == 5) {
		return '已关闭'
	}
})
