import {
	addNormalNotificationObserver,
	postNotification,
	removeNotificationObserver
} from "../../../../common/utils/notificationCenter";
import {
	readStorage,
	saveStorage
} from "../../../../common/utils/storageUtil";
import ContentFormat from "../models/ContentFormat";
import {
	apiGET,
	apiPOST
} from "../../../../common/utils/apiRequest.js"

/** 获取文章详情
 * @param {Object} bookId
 * @param {Object} callback
 */
function getBookDetailData(bookId, callback) {
	apiPOST({
		url: `/book/info`,
		data: {
			id: bookId
		},
		callback
	})
}

/** 获取推荐文章
 * @param {Object} bookId
 * @param {Object} callback
 */
function getBookRecommendData(bookId, callback) {
	apiPOST({
		url: `/book/recommend`,
		data: {
			id: bookId
		},
		callback
	})
}
/** 添加阅读记录
 * @param {Object} bookId
 * @param {Object} callback
 */
function addReadRecord(bookId, callback) {
	apiPOST({
		url: `/book/addReadRecord`,
		data: {
			articleId: bookId
		},
		callback
	})
}

// 样式设置
const KEY_STORAGE_CONTENT_FORMAT = "KEY_SOTRAGE_CONTENT_FORMAT";
const KEY_NOTIFICATION_CONTENT_FORMAT_CHANGE = "KEY_NOTIFICATION_CONTENT_FORMAT_CHANGE";

function saveContentFormat(format) {
	let result = readContentFormat();
	Object.keys(format).forEach(key => {
		let value = format[key];
		result[key] = value || result[key];
	})
	saveStorage(KEY_STORAGE_CONTENT_FORMAT, result);
	notifyContentFormatChange(new ContentFormat(result));
}

function readContentFormat() {
	let result = readStorage(KEY_STORAGE_CONTENT_FORMAT);
	if (result) {
		result = new ContentFormat(result);
	} else {
		result = new ContentFormat({
			fontSize: 17,
			backgroundColor: "#fff"
		})
	}
	return result;
}

function notifyContentFormatChange(format) {
	postNotification(KEY_NOTIFICATION_CONTENT_FORMAT_CHANGE, format);
}

function watchContentFormatChange(fn, observer) {
	if (typeof fn == 'function') fn.call(observer, readContentFormat());
	addNormalNotificationObserver(KEY_NOTIFICATION_CONTENT_FORMAT_CHANGE, (data) => {
		if (typeof fn == 'function') fn.call(observer, data);
	}, observer)
}

function removeContentFormatChangeWatch(observer) {
	removeNotificationObserver(KEY_NOTIFICATION_CONTENT_FORMAT_CHANGE, observer);
}

module.exports = {
	getBookDetailData,
	getBookRecommendData,
	addReadRecord,
	/**
	 * content format
	 */
	saveContentFormat,
	readContentFormat,
	notifyContentFormatChange,
	watchContentFormatChange,
	removeContentFormatChangeWatch
}