import {
	addNormalNotificationObserver,
	postNotification,
	removeNotificationObserver
} from "../../../../common/utils/notificationCenter";
import {
	readStorage,
	saveStorage,
	clearStorage
} from "../../../../common/utils/storageUtil";
const KEY_STORAGE_SEARCH_HISTORY = "KEY_STORAGE_SEARCH_HISTORY";
const KEY_NOTIFICATION_SEARCH_HISTORY_CHANGE = 'KEY_NOTIFICATION_SEARCH_HISTORY_CHANGE';
const MAX_SEARCH_HISTORY_LENGTH = 10;

function saveSearchHistory(keyword) {
	if (!keyword) return;
	let result = readSearchHistory();
	if (result.indexOf(keyword) != -1) return;
	if (result.length == MAX_SEARCH_HISTORY_LENGTH) {
		result.shift();
	}
	result.push(keyword);
	saveStorage(KEY_STORAGE_SEARCH_HISTORY, result);
	noticeSearchHistoryChange(result);
}

function clearSearchHistory() {
	clearStorage(KEY_STORAGE_SEARCH_HISTORY);
	noticeSearchHistoryChange([]);
}

function readSearchHistory() {
	let result = readStorage(KEY_STORAGE_SEARCH_HISTORY);
	return result || []
}

function noticeSearchHistoryChange(result) {
	if (!result) {
		result = readSearchHistory();
	}
	postNotification(KEY_NOTIFICATION_SEARCH_HISTORY_CHANGE, {
		result
	})
}

function watchSearchHistoryChange(fn, observer) {
	if (typeof fn == 'function') fn.call(observer, readSearchHistory());
	addNormalNotificationObserver(KEY_NOTIFICATION_SEARCH_HISTORY_CHANGE, (info) => {
		if (typeof fn == 'function') fn.call(observer, info.result)
	}, observer);
}

function removeSearchHistoryChangeWatch(observer) {
	removeNotificationObserver(KEY_NOTIFICATION_SEARCH_HISTORY_CHANGE, observer);
}

function getHotRecommondData(pageNo, pageSize, callback) {
	if (typeof callback == 'function') callback(true, []);
}

module.exports = {
	/**
	 * 搜索历史
	 */
	saveSearchHistory,
	readSearchHistory,
	clearSearchHistory,
	watchSearchHistoryChange,
	removeSearchHistoryChangeWatch,
	/**
	 * 热门推荐
	 */
	getHotRecommondData,
}